/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "lua_func_dia.h"

#include "lauxlib.h"
#include "lua_table_func.h"
#include "lstate.h"

#include "InterfaceGUI.h"

using namespace std;

//-----------------------------------------------------------------------------
// prototypes
//-----------------------------------------------------------------------------

static int lua_func_dia_delete(lua_State *lua);
static int lua_func_dia_curve_new(lua_State *lua);
static int lua_func_dia_curve_delete_all(lua_State *lua);
static int lua_func_dia_save_img(lua_State *lua);
static int lua_func_dia_save_csv(lua_State *lua);

static int lua_func_dia_curve_delete(lua_State *lua);
static int lua_func_dia_curve_clear_data(lua_State *lua);
static int lua_func_dia_curve_set_data(lua_State *lua);
static int lua_func_dia_curve_append_data(lua_State *lua);

//-----------------------------------------------------------------------------

int lua_func_dia_new(lua_State *lua)
{
  InterfaceGUI  *iGui = lua->iGui;
  const char    *diaName = luaL_checkstring(lua, 1);

  iGui->DiaDiagramNew(diaName);

  lua_newtable(lua);

  lua_pushstring(lua, diaName);
  lua_setfield(lua, -2, "diagram_name");

  lua_pushcfunction(lua, lua_func_dia_delete);
  lua_setfield(lua, -2, "delete");

  lua_pushcfunction(lua, lua_func_dia_curve_new);
  lua_setfield(lua, -2, "new_curve");

  lua_pushcfunction(lua, lua_func_dia_save_img);
  lua_setfield(lua, -2, "save_img");

  lua_pushcfunction(lua, lua_func_dia_save_csv);
  lua_setfield(lua, -2, "save_csv");

  lua_pushcfunction(lua, lua_func_dia_curve_delete_all);
  lua_setfield(lua, -2, "delete_all_curves");

  return 1;
}

//-----------------------------------------------------------------------------

int lua_func_dia_delete_all(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  iGui->DiaDiagramDeleteAll();

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_dia_delete(lua_State *lua)
{
  InterfaceGUI  *iGui = lua->iGui;
  QString       diaName = lua_get_table_str(lua, 1, "diagram_name");

  iGui->DiaDiagramDelete(diaName);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_dia_save_img(lua_State *lua)
{
  InterfaceGUI  *iGui = lua->iGui;
  QString       diaName = lua_get_table_str(lua, 1, "diagram_name");
  QString       fileName = luaL_checkstring(lua, 2);

  iGui->DiaDiagramSaveImg(diaName, fileName);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_dia_save_csv(lua_State *lua)
{
  InterfaceGUI  *iGui = lua->iGui;
  QString       diaName = lua_get_table_str(lua, 1, "diagram_name");
  QString       fileName = luaL_checkstring(lua, 2);

  iGui->DiaDiagramSaveCsv(diaName, fileName);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_dia_curve_delete_all(lua_State *lua)
{
  InterfaceGUI  *iGui = lua->iGui;
  QString       diaName = lua_get_table_str(lua, 1, "diagram_name");

  iGui->DiaCurveDeleteAll(diaName);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_dia_curve_new(lua_State *lua)
{
  InterfaceGUI  *iGui = lua->iGui;
  QString       diaName = lua_get_table_str(lua, 1, "diagram_name");
  const char    *curveName = luaL_checkstring(lua, 2);
  int           yAxis = 1;

  if (!lua_isnoneornil(lua, 3))
    yAxis = luaL_checkinteger(lua, 3);

  iGui->DiaCurveNew(diaName, curveName, yAxis);

  lua_newtable(lua);

  lua_getfield(lua, 1, "diagram_name");
  lua_setfield(lua, -2, "diagram_name");

  lua_pushstring(lua, curveName);
  lua_setfield(lua, -2, "curve_name");

  lua_pushcfunction(lua, lua_func_dia_curve_delete);
  lua_setfield(lua, -2, "delete");

  lua_pushcfunction(lua, lua_func_dia_curve_set_data);
  lua_setfield(lua, -2, "set_data");

  lua_pushcfunction(lua, lua_func_dia_curve_append_data);
  lua_setfield(lua, -2, "append_data");

  lua_pushcfunction(lua, lua_func_dia_curve_clear_data);
  lua_setfield(lua, -2, "clear_data");

  return 1;
}

//-----------------------------------------------------------------------------

int lua_func_dia_curve_delete(lua_State *lua)
{
  InterfaceGUI  *iGui = lua->iGui;
  QString       diaName = lua_get_table_str(lua, 1, "diagram_name");
  QString       curveName = lua_get_table_str(lua, 1, "curve_name");

  iGui->DiaCurveDelete(diaName, curveName);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_dia_curve_set_data(lua_State *lua)
{
  InterfaceGUI  *iGui = lua->iGui;
  QString       diaName = lua_get_table_str(lua, 1, "diagram_name");
  QString       curveName = lua_get_table_str(lua, 1, "curve_name");

  luaL_checktype(lua, 2, LUA_TTABLE);

  QVector <double> vx, vy;

  if (lua_istable(lua, 3))
  {
    int idx = 1;

    for (lua_pushnil(lua); lua_next(lua, 2); lua_pop(lua, 1))
    {
      double x = luaL_checknumber(lua, -1);

      lua_rawgeti(lua, 3, idx);
      double y = luaL_checknumber(lua, -1);
      lua_pop(lua, 1);

      vx.push_back(x);
      vy.push_back(y);

      idx++;
    }
  }
  else
  {
    for (lua_pushnil(lua); lua_next(lua, 2); lua_pop(lua, 1))
    {
      luaL_checktype(lua, -1, LUA_TTABLE);

      lua_rawgeti(lua, -1, 1);
      double x = luaL_checknumber(lua, -1);
      lua_pop(lua, 1);

      lua_rawgeti(lua, -1, 2);
      double y = luaL_checknumber(lua, -1);
      lua_pop(lua, 1);

      vx.push_back(x);
      vy.push_back(y);
    }
  }

  iGui->DiaCurveSetData(diaName, curveName, vx, vy);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_dia_curve_append_data(lua_State *lua)
{
  InterfaceGUI  *iGui = lua->iGui;
  QString       diaName = lua_get_table_str(lua, 1, "diagram_name");
  QString       curveName = lua_get_table_str(lua, 1, "curve_name");

  double x = 0;
  double y = 0;

  if (lua_istable(lua, 2))
  {
    lua_rawgeti(lua, 2, 1);
    x = luaL_checknumber(lua, -1);
    lua_pop(lua, 1);

    lua_rawgeti(lua, 2, 2);
    y = luaL_checknumber(lua, -1);
    lua_pop(lua, 1);
  }
  else
  {
    x = luaL_checknumber(lua, 2);
    y = luaL_checknumber(lua, 3);
  }

  iGui->DiaCurveAppendData(diaName, curveName, x, y);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_dia_curve_clear_data(lua_State *lua)
{
  InterfaceGUI  *iGui = lua->iGui;
  QString       diaName = lua_get_table_str(lua, 1, "diagram_name");
  QString       curveName = lua_get_table_str(lua, 1, "curve_name");

  iGui->DiaCurveClearData(diaName, curveName);

  return 0;
}

//-----------------------------------------------------------------------------
