/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "lua_table_func.h"

#include <string.h>
#include <stdlib.h>

#include "lua_helper.h"

//-----------------------------------------------------------------------------
// prototypes

const lua_table_property * lua_table_get_property(lua_State *lua, int table_idx);
void lua_table_register_functions(lua_State *lua, const lua_table_function *func_list);
int lua_table_info_functions(lua_State *lua, const char *base_name);

//-----------------------------------------------------------------------------

const lua_table_property * lua_table_get_property(lua_State *lua, int table_idx)
{
  if (luaL_getmetafield(lua, table_idx, "prop_list"))
  {
    const lua_table_property *prop_list = (const lua_table_property *)lua_touserdata(lua, -1);
    lua_pop(lua, 1);

    if (prop_list)
    {
      const char *name = lua_tostring(lua, table_idx + 1);

      while (prop_list && prop_list->name)
      {
        if (strcmp(name, prop_list->name) == 0)
          return prop_list;

        prop_list++;
      }
      
      lua_throw_error_from_c(lua, "Table has no property or function \"%s\".", name);
    }
  }

  return 0;
}

//-----------------------------------------------------------------------------

static int lua_table_read_access(lua_State *lua)
{
  const lua_table_property *property = lua_table_get_property(lua, -2);
  if (property)
  {
    if (property->read)
    {
      property->read(lua);
      return 1;
    }
    else
    {
      lua_throw_error_from_c(lua, "Can not read table property %s because it is write only!", property->name);
    }
  }

  return 0;
}

//-----------------------------------------------------------------------------

static int lua_table_write_access(lua_State *lua)
{
  const lua_table_property *property = lua_table_get_property(lua, -3);
  if (property)
  {
    if (property->write)
    {
      property->write(lua);
    }
    else
    {
      lua_throw_error_from_c(lua, "Can not write table property %s because it is read only!", property->name);
    }
  }

  return 0;
}

//-----------------------------------------------------------------------------

static int lua_table_tostring(lua_State *lua)
{
  return luaL_getmetafield(lua, -1, "table_name");
}

//-----------------------------------------------------------------------------

void lua_table_register_functions(lua_State *lua,
                                  const lua_table_function *func_list)
{
  const lua_table_function *f;

  for (f = func_list; f && f->name; f++)
  {
    lua_pushcfunction(lua, f->func);
    lua_setfield(lua, -2, f->name);
  }
}

//-----------------------------------------------------------------------------

void lua_global_table_begin(lua_State *lua,
                            const char *table_name)
{
  (void)table_name;

  // create table
  lua_newtable(lua);
}

//-----------------------------------------------------------------------------

void lua_global_table_end(lua_State *lua,
                          const char *table_name,
                          const lua_table_function *func_list,
                          const lua_table_property *prop_list)
{
  // register functions
  lua_table_register_functions(lua, func_list);

  // create metatable
  lua_newtable(lua);

  // function for property read access
  lua_pushstring(lua, "__index");
  lua_pushcfunction(lua, lua_table_read_access);
  lua_settable(lua, -3);

  // function for property write access
  lua_pushstring(lua, "__newindex");
  lua_pushcfunction(lua, lua_table_write_access);
  lua_settable(lua, -3);

  // function for table to string conversion
  lua_pushstring(lua, "__tostring");
  lua_pushcfunction(lua, lua_table_tostring);
  lua_settable(lua, -3);

  // store table name
  lua_pushstring(lua, "table_name");
  lua_pushlightuserdata(lua, (void *)table_name);
  lua_settable(lua, -3);

  // store table property list
  lua_pushstring(lua, "prop_list");
  lua_pushlightuserdata(lua, (void *)prop_list);
  lua_settable(lua, -3);

  // store table function list
  lua_pushstring(lua, "func_list");
  lua_pushlightuserdata(lua, (void *)func_list);
  lua_settable(lua, -3);

  // set metatable
  lua_setmetatable(lua, -2);

  // set global table
  lua_setglobal(lua, table_name);
}

//-----------------------------------------------------------------------------

void lua_sub_table_begin(lua_State *lua,
                         const char *table_name)
{
  // push table name
  lua_pushstring(lua, table_name);

  // create table
  lua_newtable(lua);
}

//-----------------------------------------------------------------------------

void lua_sub_table_end(lua_State *lua,
                       const char *table_name,
                       const lua_table_function *func_list,
                       const lua_table_property *prop_list)
{
  (void)table_name;

  // register functions
  lua_table_register_functions(lua, func_list);

  // create metatable
  lua_newtable(lua);

  // function for property read access
  lua_pushstring(lua, "__index");
  lua_pushcfunction(lua, lua_table_read_access);
  lua_settable(lua, -3);

  // function for property write access
  lua_pushstring(lua, "__newindex");
  lua_pushcfunction(lua, lua_table_write_access);
  lua_settable(lua, -3);

  // function for table to string conversion
  lua_pushstring(lua, "__tostring");
  lua_pushcfunction(lua, lua_table_tostring);
  lua_settable(lua, -3);

  // store table name
  lua_pushstring(lua, "table_name");
  lua_pushlightuserdata(lua, (void *)table_name);
  lua_settable(lua, -3);

  // store table property list
  lua_pushstring(lua, "prop_list");
  lua_pushlightuserdata(lua, (void *)prop_list);
  lua_settable(lua, -3);

  // store table function list
  lua_pushstring(lua, "func_list");
  lua_pushlightuserdata(lua, (void *)func_list);
  lua_settable(lua, -3);

  // set metatable
  lua_setmetatable(lua, -2);

  // set sub table
  lua_settable(lua, -3);
}

//-----------------------------------------------------------------------------

void lua_set_global_function(lua_State *lua, lua_CFunction func, const char *funcName)
{
  lua_pushcfunction(lua, func);
  lua_setglobal(lua, funcName);
}

//-----------------------------------------------------------------------------

QString lua_get_table_str(lua_State *lua, int tableIdx, const char *fieldName)
{
  luaL_checktype(lua, tableIdx, LUA_TTABLE);
  lua_getfield(lua, tableIdx, fieldName);

  QString str = luaL_checkstring(lua, -1);

  lua_pop(lua, 1);

  return str;
}

//-----------------------------------------------------------------------------
