/*
  modified and extended by Torsten Brischalle

  based upon qterminalwidget from http://code.google.com/p/qterminalwidget

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "qconsolewidget.h"

#include <QKeyEvent>

#include <iostream>

using namespace std;

//-----------------------------------------------------------------------------

QConsoleWidget::QConsoleWidget(QWidget *parent)
: QPlainTextEdit(parent)
{
  m_luaThread = 0;
  m_fixedPosition = 0;
  m_historyIter = m_history.end();

  setUndoRedoEnabled(false);
  document()->setMaximumBlockCount(CONSOLE_MAX_NUM_LINES);
}

//-----------------------------------------------------------------------------

QConsoleWidget::~QConsoleWidget()
{
}

//-----------------------------------------------------------------------------

void QConsoleWidget::AttachLuaThread(LuaThread *luaThread)
{
  m_luaThread = luaThread;

  connect(m_luaThread, SIGNAL(LuaStarted()), this, SLOT(LuaStarted()));
  connect(m_luaThread, SIGNAL(LuaStopped()), this, SLOT(LuaStopped()));
  connect(this, SIGNAL(SignalWrite(QString)), this, SLOT(SlotWrite(QString)));

  Write(CONSOLE_PROMPT);
}

//-----------------------------------------------------------------------------

void QConsoleWidget::LuaStarted()
{
}

//-----------------------------------------------------------------------------

void QConsoleWidget::LuaStopped()
{
  Write(CONSOLE_PROMPT);
}

//-----------------------------------------------------------------------------

void QConsoleWidget::Write(QString szOutput)
{
  emit SignalWrite(szOutput);
}

//-----------------------------------------------------------------------------

void QConsoleWidget::SlotWrite(QString szOutput)
{
  WriteImpl(szOutput);
}

//-----------------------------------------------------------------------------

void QConsoleWidget::WriteImpl(QString szOutput)
{
  QTextCursor c = textCursor();
  c.movePosition(QTextCursor::End);
  setTextCursor(c);

  insertPlainText(szOutput);
  m_fixedPosition = textCursor().position();
  ensureCursorVisible();
}

//-----------------------------------------------------------------------------

void QConsoleWidget::WriteHistory(QString szOutput)
{
  QTextCursor c = textCursor();

  c.setPosition(m_fixedPosition, QTextCursor::MoveAnchor);
  c.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
  c.removeSelectedText();

  c.movePosition(QTextCursor::End);
  setTextCursor(c);

  insertPlainText(szOutput);
  ensureCursorVisible();
}

//-----------------------------------------------------------------------------

void QConsoleWidget::keyPressEvent(QKeyEvent *event)
{
  bool  accept = false;
  int   key = event->key();

  if (key == Qt::Key_Backspace || event->key() == Qt::Key_Left)
  {
    accept = textCursor().position() > m_fixedPosition;
  }
  else if (key == Qt::Key_Return)
  {
    QString txt = toPlainText();
    int     count = txt.count() - m_fixedPosition;
    QString cmd = txt.right(count);

    if ((m_history.size() == 0) || (m_history.back() != cmd))
    {
      m_history.push_back(cmd);
      while (m_history.size() > CONSOLE_MAX_HISTORY)
        m_history.pop_front();

      m_historyIter = m_history.end();
    }

    if (m_luaThread->IsRunning())
      m_luaThread->PutToStdInput(cmd + "\n");
    else
      m_luaThread->Execute(cmd);

    accept = false;
    WriteImpl("\n");
  }
  else if (key == Qt::Key_Up)
  {
    if ((m_history.size() > 0) && !m_luaThread->IsRunning())
    {
      if (m_historyIter != m_history.begin())
        m_historyIter--;

      WriteHistory(*m_historyIter);
    }
  }
  else if (key == Qt::Key_Down)
  {
    if ((m_history.size() > 0) && !m_luaThread->IsRunning())
    {
      if (m_historyIter != m_history.end())
        m_historyIter++;

      if (m_historyIter != m_history.end())
        WriteHistory(*m_historyIter);
      else
        WriteHistory(QString());
    }
  }
  else if (key == Qt::Key_Home)
  {
    QTextCursor c = textCursor();
    c.setPosition(m_fixedPosition, QTextCursor::MoveAnchor);
    setTextCursor(c);
  }
  else if ((key == Qt::Key_C) && (event->modifiers() & Qt::ControlModifier))
  {
    accept = true;
  }
  else
  {
    accept = (textCursor().position() >= m_fixedPosition) || m_luaThread->IsRunning();
  }

  if (accept)
  {
    QPlainTextEdit::keyPressEvent(event);
  }
}

//-----------------------------------------------------------------------------

void QConsoleWidget::cursorPositionChanged()
{
  if (textCursor().position() < m_fixedPosition)
  {
    textCursor().setPosition(m_fixedPosition);
  }
}

//-----------------------------------------------------------------------------

