/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "variablelist.h"

using namespace std;

//-----------------------------------------------------------------------------

VariableList::VariableList()
{
}

//-----------------------------------------------------------------------------

void VariableList::Clear()
{
  m_varList.clear();
}

//-----------------------------------------------------------------------------

void VariableList::GetGlobalVariables(lua_State *lua)
{
  for (lua_pushnil(lua); lua_next(lua, LUA_GLOBALSINDEX); lua_pop(lua, 1))
  {
    if (lua_type(lua, -1) != LUA_TFUNCTION)
    {
      Variable var;

      lua_pushvalue(lua, -2);
      var.m_name = lua_tostring(lua, -1);
      lua_pop(lua, 1);

      var.m_type = lua_typename(lua, lua_type(lua, -1));
      var.m_value = LuaValueToQString(lua, -1);

      if (lua_type(lua, -1) == LUA_TTABLE)
      {
        switch (lua_type(lua, -2))
        {
        case LUA_TNUMBER:
          var.m_key = lua_tonumber(lua, -2);
          break;
        case LUA_TSTRING:
          var.m_key = var.m_name;
          break;
        }
      }

      m_varList.push_back(var);
    }
  }
}

//-----------------------------------------------------------------------------

void VariableList::GetLocalVariables(lua_State *lua)
{
  lua_Debug   ar;
  int         idx = 1;
  const char  *varName;

  lua_getstack(lua, 0, &ar);

  varName = lua_getlocal(lua, &ar, idx);
  while (varName)
  {
    if (varName[0] != '(')
    {
      Variable var;

      var.m_name = varName;
      var.m_type = lua_typename(lua, lua_type(lua, -1));
      var.m_value = LuaValueToQString(lua, -1);

      if (lua_type(lua, -1) == LUA_TTABLE)
        var.m_key = var.m_name;

      m_varList.push_back(var);
    }

    lua_pop(lua, 1);

    idx++;
    varName = lua_getlocal(lua, &ar, idx);
  }
}

//-----------------------------------------------------------------------------

QString VariableList::LuaValueToQString(lua_State *lua, int idx)
{
  switch (lua_type(lua, idx))
  {
  case LUA_TNIL:
    return "nil";

  case LUA_TBOOLEAN:
    return lua_toboolean(lua, idx) ? "true" : "false";

  case LUA_TNUMBER:
    return QString::number(lua_tonumber(lua, idx));

  case LUA_TSTRING:
    return lua_tostring(lua, idx);

  case LUA_TTABLE:
  case LUA_TFUNCTION:
  case LUA_TTHREAD:
  case LUA_TUSERDATA:
  case LUA_TLIGHTUSERDATA:
  {
    qulonglong ptr = (qulonglong)lua_topointer(lua, idx);
    return QString("0x") + QString::number(ptr, 16);
  }

  case LUA_TNONE:
  default:
    return QString();
  }
}

//-----------------------------------------------------------------------------

void VariableList::GetLocalVariablesFromPath(lua_State *lua, const QVariantList & path)
{
  lua_Debug   ar;
  int         idx = 1;
  const char  *varName;
  QString     baseName = path[0].toString();

  lua_getstack(lua, 0, &ar);

  varName = lua_getlocal(lua, &ar, idx);
  while (varName)
  {
    if (baseName == varName)
      break;

    lua_pop(lua, 1);

    idx++;
    varName = lua_getlocal(lua, &ar, idx);
  }

  if (!varName)
    return;

  GetVariablesFromPath(lua, path);
}

//-----------------------------------------------------------------------------

void VariableList::GetGlobalVariablesFromPath(lua_State *lua, const QVariantList & path)
{
  QString     baseName = path[0].toString();
  string      s;

  s = baseName.toStdString();
  lua_pushstring(lua, s.c_str());
  lua_gettable(lua, LUA_GLOBALSINDEX);

  GetVariablesFromPath(lua, path);
}

//-----------------------------------------------------------------------------

void VariableList::GetVariablesFromPath(lua_State *lua, const QVariantList & path)
{
  int   cnt = 1;
  int   idx;

  for (idx = 1; idx < path.size(); idx++)
  {
    if (!lua_istable(lua, -1))
    {
      lua_pop(lua, cnt);
      return;
    }

    QVariant key = path[idx];

    if (key.type() == QVariant::String)
    {
      string s = key.toString().toStdString();
      lua_pushstring(lua, s.c_str());
    }
    else if (key.type() == QVariant::Double)
    {
      double d = key.toDouble();
      lua_pushnumber(lua, d);
    }
    else
    {
      lua_pop(lua, cnt);
      return;
    }

    lua_gettable(lua, -2);
    cnt++;
  }

  if (!lua_istable(lua, -1))
  {
    lua_pop(lua, cnt);
    return;
  }

  for (lua_pushnil(lua); lua_next(lua, -2); lua_pop(lua, 1))
  {
    Variable var;

    lua_pushvalue(lua, -2);
    var.m_name = lua_tostring(lua, -1);
    lua_pop(lua, 1);

    var.m_type = lua_typename(lua, lua_type(lua, -1));
    var.m_value = LuaValueToQString(lua, -1);

    if (lua_type(lua, -1) == LUA_TTABLE)
    {
      switch (lua_type(lua, -2))
      {
      case LUA_TNUMBER:
        var.m_key = lua_tonumber(lua, -2);
        break;
      case LUA_TSTRING:
        var.m_key = var.m_name;
        break;
      }
    }

    m_varList.push_back(var);
  }

  lua_pop(lua, cnt);
}

//-----------------------------------------------------------------------------
